<?php namespace EscapeWork\Manager\Banners\Repositories;

use EscapeWork\LaravelHelpers\BaseRepository;
use EscapeWork\Manager\Banners\Models\Banner;
use EscapeWork\Resize\Resize;
use Symfony\Component\HttpFoundation\File\UploadedFile;
use Exception;
use Config;
use Str;
use Log;

class BannerRepository extends BaseRepository
{

    public function __construct(Banner $banner)
    {
        $this->model = $banner;
    }

    public function getAcceptedImages()
    {
        return $this->model->acceptedImages;
    }

    public function filterAndFetch($params)
    {
        $query = $this->model->with('group');

        if (isset($params['banner_group_id']) && $params['banner_group_id'] !== '') {
            $query->where('banner_group_id', '=', $params['banner_group_id']);
        }

        if (! isset($params['expired']) || $params['expired'] != 1) {
            $query->notExpired();
        }

        return $query->orderBy('order')->get();
    }

    /**
     * Fazendo o upload do banner
     */
    public function uploadBanner($banner, $update = false)
    {
        if (! $banner instanceof UploadedFile) {
            $this->addMessage('banner', 'Selecione um arquivo');
            return false;
        }

        # validando o diretório
        if (! $this->_makeDirectory()) {
            $this->addMessage('banner', 'O diretório <strong>'.$this->getDir().'</strong> precisa de permissão de escrita para salvar os banners');

            return false;
        }

        # validando o tipo
        if (! $this->_validateBannerType($banner->getMimeType())) {
            $this->addMessage('banner', 'Apenas imagens <strong>jpg e png</strong> são aceitas');

            return false;
        }

        # formatando o nome
        try {
            $dir      = $this->getDir();
            $filename = $this->_formatName($banner->getClientOriginalName(), $dir, $banner->guessExtension());
        } catch (Exception $e) {
            $this->addMessage('banner', 'Houve um erro com a extensão de sua imagem. Por favor, verifique se a imagem possui a extensão correta e/ou se a mesma não está em CMYK.');

            return false;
        }

        # copiando a imagem
        try {
            $banner = $banner->move($dir . '/', $filename);
            $this->createSizes($dir, $filename, $banner->getMimeType());

            # excluindo o arquivo atual
            if ($update) {
                $this->_deleteBanner();
            }

            $this->banner = $filename;

            return true;
        } catch (Exception $e) {
            $this->addMessage('banner', 'Ocorreu um erro não identificado ao subir o seu banner. Por favor, tente novamente. Caso o erro persista, entre em contato com o administrador do site.');
            Log::error($e->getMessage());

            return false;
        }
    }

    /**
     * Verificando se a imagem é um gif
     */
    protected function isGif($mime)
    {
         return $mime === 'image/gif';
    }

    /**
     * Validando o tipo da imagem
     */
    public function _validateBannerType($type)
    {
        return in_array($type, $this->getAcceptedImages());
    }

    /**
     * Formatando o nome
     */
    protected function _formatName($name, $dir, $extension)
    {
        $name    = explode('.', $name);
        $total   = count($name);

        if ($total > 1) {
            unset($name[$total - 1]);
        }

        $name     = implode('', $name);
        $filename = Str::slug($name) . '.' . $extension;

        while (is_file($dir . '/' . $filename)) {
            $filename = '_' . $filename;
        }

        return $filename;
    }

    /**
     * Fazendo o resize da imagem
     *
     * @throws  EscapeWork\Resize\ResizeException
     */
    public function createSizes($dir, $filename, $mimeType = null)
    {
        $sizes = $this->getGroupSizes();

        foreach ($sizes as $size) {
            $prefix = $size->prefix == 'default' ? '' : $size->prefix . '-';
            $banner = $dir . '/' . $prefix . $filename;

            # copiando o arquivo
            if (! is_file($banner)) {
                copy($dir . '/' . $filename, $banner);
            }

            # resize
            $bannerSize = getimagesize($banner);

            if ($size->width == $bannerSize[0] && $size->height == $bannerSize[1]) {
                continue;
            }

            if (! $this->isGif($mimeType)) {
                $resize = new Resize($banner);
                $resize->setWidth($size->width)->setHeight($size->height)->crop();
            }
        }

        # size for manager
        $banner = $dir . '/' . 'manager-' . $filename;

        if (! is_file($banner)) {
            copy($dir . '/' . $filename, $banner);
        }

        # resize
        $resize = new Resize($banner);
        $resize->setWidth(200)->setHeight(75)->crop();
    }

    /**
     * Returning the group sizes
     */
    public function getGroupSizes()
    {
        $defaultSizes = Config::get('manager-banners::sizes.default');

        if (is_null($this->model->banner_group_id)) {
            return json_decode(json_encode($defaultSizes), false);
        }

        return $this->model->group->sizes->count() > 0 ? $this->model->group->sizes : $defaultSizes;
    }

    /**
     * Excluindo o arquivo antigo
     */
    public function _deleteBanner()
    {
        $dir  = $this->getDir();
        $file = $dir . '/' . $this->banner;

        if (is_file($file)) {
            unlink($file);
        }
    }

    /**
     * Criando o diretório caso não existe
     */
    public function _makeDirectory()
    {
        $dir = $this->getDir();

        if (! is_dir($dir)) {
            if (mkdir($dir, 0773)) {
                chmod($dir, 0773);
            } else {
                return false;
            }
        }

        return true;
    }

    /**
     * Excluindo o banner
     */
    public function delete()
    {
        $this->_deleteBanner();

        return $this->model->delete();
    }

    /**
     * Retornando a URL da imagem
     */
    public function getUrl()
    {
        return $this->model->getUrl();
    }

    /**
     * Retornando a URL da imagem
     */
    public function getDir()
    {
        return $this->model->getDir();
    }

    public function getActives()
    {
        return $this->model
                    ->active()
                    ->notExpired()
                    ->orderBy('order')
                    ->get();
    }
}
